@@ -7,6 +7,7 @@
 
 struct view;
 struct server;
+struct cursor_context;
 
 struct action {
 	struct wl_list link; /*
@@ -34,8 +35,16 @@ void action_arg_from_xml_node(struct action *action, const char *nodename, const
 
 bool actions_contain_toggle_keybinds(struct wl_list *action_list);
 
+/**
+ * actions_run() - Run actions.
+ * @activator: Target view to apply actions (e.g. Maximize, Focus etc.).
+ * NULL is allowed, in which case the focused/hovered view is used.
+ * @ctx: Set for action invocations via mousebindings. Used to get the
+ * direction of resize or the position of the window menu button for ShowMenu
+ * action.
+ */
 void actions_run(struct view *activator, struct server *server,
-	struct wl_list *actions, uint32_t resize_edges);
+	struct wl_list *actions, struct cursor_context *ctx);
 
 void action_free(struct action *action);
 void action_list_free(struct wl_list *action_list);
