@@ -44,23 +44,10 @@ struct theme {
 	int window_titlebar_padding_width;
 	int window_titlebar_padding_height;
 
-	int title_height;
-	int menu_overlap_x;
-	int menu_overlap_y;
-
-	/* colors */
-	float window_active_border_color[4];
-	float window_inactive_border_color[4];
+	int titlebar_height;
 
 	float window_toggled_keybinds_color[4];
-
-	float window_active_title_bg_color[4];
-	float window_inactive_title_bg_color[4];
-
-	float window_active_label_text_color[4];
-	float window_inactive_label_text_color[4];
 	enum lab_justification window_label_text_justify;
-	enum lab_justification menu_title_text_justify;
 
 	/* buttons */
 	int window_button_width;
@@ -70,25 +57,67 @@ struct theme {
 	/* the corner radius of the hover effect */
 	int window_button_hover_bg_corner_radius;
 
-	int menu_item_padding_x;
-	int menu_item_padding_y;
-	int menu_item_height;
+	/*
+	 * Themes/textures for each active/inactive window. Indexed by
+	 * THEME_INACTIVE and THEME_ACTIVE.
+	 */
+	struct {
+		/* TODO: add toggled/hover/pressed/disabled colors for buttons */
+		float button_colors[LAB_SSD_BUTTON_LAST + 1][4];
 
+		float border_color[4];
+		float toggled_keybinds_color[4];
+		float title_bg_color[4];
+		float label_text_color[4];
+
+		/* window drop-shadows */
+		int shadow_size;
+		float shadow_color[4];
+
+		/*
+		 * The texture of a window buttons for each hover/toggled/rounded
+		 * state. This can be accessed like:
+		 *
+		 * buttons[LAB_SSD_BUTTON_ICONIFY][LAB_BS_HOVERD | LAB_BS_TOGGLED]
+		 *
+		 * Elements in buttons[0] are all NULL since LAB_SSD_BUTTON_FIRST is 1.
+		 */
+		struct lab_data_buffer *buttons
+			[LAB_SSD_BUTTON_LAST + 1][LAB_BS_ALL + 1];
+
+		struct lab_data_buffer *corner_top_left_normal;
+		struct lab_data_buffer *corner_top_right_normal;
+
+		struct lab_data_buffer *shadow_corner_top;
+		struct lab_data_buffer *shadow_corner_bottom;
+		struct lab_data_buffer *shadow_edge;
+	} window[2];
+
+	/* Derived from font sizes */
+	int menu_item_height;
+	int menu_header_height;
+
+	int menu_overlap_x;
+	int menu_overlap_y;
+	int menu_min_width;
+	int menu_max_width;
+	int menu_border_width;
+	float menu_border_color[4];
+
+	int menu_items_padding_x;
+	int menu_items_padding_y;
 	float menu_items_bg_color[4];
 	float menu_items_text_color[4];
 	float menu_items_active_bg_color[4];
 	float menu_items_active_text_color[4];
 
-	int menu_min_width;
-	int menu_max_width;
-
 	int menu_separator_line_thickness;
 	int menu_separator_padding_width;
 	int menu_separator_padding_height;
 	float menu_separator_color[4];
 
 	float menu_title_bg_color[4];
-
+	enum lab_justification menu_title_text_justify;
 	float menu_title_text_color[4];
 
 	int osd_border_width;
@@ -112,45 +141,6 @@ struct theme {
 	struct theme_snapping_overlay
 		snapping_overlay_region, snapping_overlay_edge;
 
-	/* window drop-shadows */
-	int window_active_shadow_size;
-	int window_inactive_shadow_size;
-	float window_active_shadow_color[4];
-	float window_inactive_shadow_color[4];
-
-	struct {
-		/*
-		 * The texture of a window buttons for each hover/toggled/rounded
-		 * state. This can be accessed like:
-		 *
-		 * buttons[LAB_SSD_BUTTON_ICONIFY][LAB_BS_HOVERD | LAB_BS_TOGGLED]
-		 *
-		 * Elements in buttons[0] are all NULL since LAB_SSD_BUTTON_FIRST is 1.
-		 */
-		struct lab_data_buffer *buttons
-			[LAB_SSD_BUTTON_LAST + 1][LAB_BS_ALL + 1];
-
-		/* TODO: add toggled/hover/pressed/disabled colors for buttons */
-		float button_colors[LAB_SSD_BUTTON_LAST + 1][4];
-
-		/* TODO: move other window.(in)active.* entries to here */
-
-	} window[2]; /* indexed by THEME_INACTIVE and THEME_ACTIVE */
-
-	/* textures */
-
-	struct lab_data_buffer *corner_top_left_active_normal;
-	struct lab_data_buffer *corner_top_right_active_normal;
-	struct lab_data_buffer *corner_top_left_inactive_normal;
-	struct lab_data_buffer *corner_top_right_inactive_normal;
-
-	struct lab_data_buffer *shadow_corner_top_active;
-	struct lab_data_buffer *shadow_corner_bottom_active;
-	struct lab_data_buffer *shadow_edge_active;
-	struct lab_data_buffer *shadow_corner_top_inactive;
-	struct lab_data_buffer *shadow_corner_bottom_inactive;
-	struct lab_data_buffer *shadow_edge_inactive;
-
 	/*
 	 * Not set in rc.xml/themerc, but derived from the tallest titlebar
 	 * object plus 2 * window_titlebar_padding_height
