@@ -57,6 +57,11 @@ enum view_axis {
 	VIEW_AXIS_HORIZONTAL = (1 << 0),
 	VIEW_AXIS_VERTICAL = (1 << 1),
 	VIEW_AXIS_BOTH = (VIEW_AXIS_HORIZONTAL | VIEW_AXIS_VERTICAL),
+	/*
+	 * If view_axis is treated as a bitfield, INVALID should never
+	 * set the HORIZONTAL or VERTICAL bits.
+	 */
+	VIEW_AXIS_INVALID = (1 << 2),
 };
 
 enum view_edge {
@@ -106,6 +111,7 @@ enum window_type {
 
 struct view;
 struct wlr_surface;
+struct foreign_toplevel;
 
 /* Common to struct view and struct xwayland_unmanaged */
 struct mappable {
@@ -260,16 +266,6 @@ struct view {
 		struct multi_rect *rect;
 	} resize_outlines;
 
-	struct foreign_toplevel {
-		struct wlr_foreign_toplevel_handle_v1 *handle;
-		struct wl_listener maximize;
-		struct wl_listener minimize;
-		struct wl_listener fullscreen;
-		struct wl_listener activate;
-		struct wl_listener close;
-		struct wl_listener destroy;
-	} toplevel;
-
 	struct mappable mappable;
 
 	struct wl_listener destroy;
@@ -281,6 +277,18 @@ struct view {
 	struct wl_listener request_maximize;
 	struct wl_listener request_fullscreen;
 	struct wl_listener set_title;
+
+	struct foreign_toplevel *foreign_toplevel;
+
+	struct {
+		struct wl_signal new_app_id;
+		struct wl_signal new_title;
+		struct wl_signal new_outputs;
+		struct wl_signal maximized;
+		struct wl_signal minimized;
+		struct wl_signal fullscreened;
+		struct wl_signal activated;     /* bool *activated */
+	} events;
 };
 
 struct view_query {
@@ -290,6 +298,16 @@ struct view_query {
 	int window_type;
 	char *sandbox_engine;
 	char *sandbox_app_id;
+	enum three_state shaded;
+	enum view_axis maximized;
+	enum three_state iconified;
+	enum three_state focused;
+	enum three_state omnipresent;
+	enum view_edge tiled;
+	char *tiled_region;
+	char *desktop;
+	enum ssd_mode decoration;
+	char *monitor;
 };
 
 struct xdg_toplevel_view {
@@ -363,11 +381,27 @@ bool view_matches_query(struct view *view, struct view_query *query);
  *		printf("%s\n", view_get_string_prop(view, "app_id"));
  *	}
  */
-#define for_each_view(view, head, criteria)		\
-	for (view = view_next(head, NULL, criteria);	\
-	     view;					\
+#define for_each_view(view, head, criteria)           \
+	for (view = view_next(head, NULL, criteria);  \
+	     view;                                    \
 	     view = view_next(head, view, criteria))
 
+/**
+ * for_each_view_reverse() - iterate over all views which match criteria
+ * @view: Iterator.
+ * @head: Head of list to iterate over.
+ * @criteria: Criteria to match against.
+ * Example:
+ *	struct view *view;
+ *	for_each_view_reverse(view, &server->views, LAB_VIEW_CRITERIA_NONE) {
+ *		printf("%s\n", view_get_string_prop(view, "app_id"));
+ *	}
+ */
+#define for_each_view_reverse(view, head, criteria)   \
+	for (view = view_prev(head, NULL, criteria);  \
+	     view;                                    \
+	     view = view_prev(head, view, criteria))
+
 /**
  * view_next() - Get next view which matches criteria.
  * @head: Head of list to iterate over.
@@ -380,6 +414,18 @@ bool view_matches_query(struct view *view, struct view_query *query);
 struct view *view_next(struct wl_list *head, struct view *view,
 	enum lab_view_criteria criteria);
 
+/**
+ * view_prev() - Get previous view which matches criteria.
+ * @head: Head of list to iterate over.
+ * @view: Current view from which to find the previous one. If NULL is provided
+ *        as the view argument, the end of the list will be used.
+ * @criteria: Criteria to match against.
+ *
+ * Returns NULL if there are no views matching the criteria.
+ */
+struct view *view_prev(struct wl_list *head, struct view *view,
+	enum lab_view_criteria criteria);
+
 /*
  * Same as `view_next()` except that they iterate one whole cycle rather than
  * stopping at the list-head
@@ -565,6 +611,8 @@ void view_adjust_size(struct view *view, int *w, int *h);
 void view_evacuate_region(struct view *view);
 void view_on_output_destroy(struct view *view);
 void view_connect_map(struct view *view, struct wlr_surface *surface);
+
+void view_init(struct view *view);
 void view_destroy(struct view *view);
 
 enum view_axis view_axis_parse(const char *direction);
