@@ -107,19 +107,33 @@ lab_wlr_scene_output_commit(struct wlr_scene_output *scene_output,
 		return false;
 	}
 
+	if (state->tearing_page_flip) {
+		if (!wlr_output_test_state(wlr_output, state)) {
+			state->tearing_page_flip = false;
+		}
+	}
+
 	struct wlr_box additional_damage = {0};
 	if (state->buffer && is_magnify_on()) {
 		magnify(output, state->buffer, &additional_damage);
 	}
 
-	if (state == &output->pending) {
-		if (!wlr_output_commit(wlr_output)) {
-			wlr_log(WLR_INFO, "Failed to commit output %s",
-				wlr_output->name);
-			return false;
+	bool committed = wlr_output_commit_state(wlr_output, state);
+	/*
+	 * Handle case where the ouput state test for tearing succeeded,
+	 * but actual commit failed. Retry wihout tearing.
+	 */
+	if (!committed && state->tearing_page_flip) {
+		state->tearing_page_flip = false;
+		committed = wlr_output_commit_state(wlr_output, state);
+	}
+	if (committed) {
+		if (state == &output->pending) {
+			wlr_output_state_finish(&output->pending);
+			wlr_output_state_init(&output->pending);
 		}
-	} else if (!wlr_output_commit_state(wlr_output, state)) {
-		wlr_log(WLR_INFO, "Failed to commit state for output %s",
+	} else {
+		wlr_log(WLR_INFO, "Failed to commit output %s",
 			wlr_output->name);
 		return false;
 	}
