@@ -15,6 +15,7 @@
 #include "osd.h"
 #include "regions.h"
 #include "view.h"
+#include "window-rules.h"
 #include "workspaces.h"
 
 enum lab_key_handled {
@@ -80,6 +81,43 @@ end_cycling(struct server *server)
 	should_cancel_cycling_on_next_key_release = false;
 }
 
+static struct wlr_seat_client *
+seat_client_from_keyboard_resource(struct wl_resource *resource)
+{
+	return wl_resource_get_user_data(resource);
+}
+
+static void
+broadcast_modifiers_to_unfocused_clients(struct wlr_seat *seat,
+		const struct wlr_keyboard_modifiers *modifiers)
+{
+	struct wlr_seat_client *client;
+	wl_list_for_each(client, &seat->clients, link) {
+		if (client == seat->keyboard_state.focused_client) {
+			/*
+			 * We've already notified the focused client by calling
+			 * wlr_seat_keyboard_notify_modifiers()
+			 */
+			continue;
+		}
+		uint32_t serial = wlr_seat_client_next_serial(client);
+		struct wl_resource *resource;
+		wl_resource_for_each(resource, &client->keyboards) {
+			if (!seat_client_from_keyboard_resource(resource)) {
+				continue;
+			}
+			if (!modifiers) {
+				wl_keyboard_send_modifiers(resource, serial, 0,
+					0, 0, 0);
+			} else {
+				wl_keyboard_send_modifiers(resource, serial,
+					modifiers->depressed, modifiers->latched,
+					modifiers->locked, modifiers->group);
+			}
+		}
+	}
+}
+
 static void
 keyboard_modifiers_notify(struct wl_listener *listener, void *data)
 {
@@ -112,8 +150,31 @@ keyboard_modifiers_notify(struct wl_listener *listener, void *data)
 	}
 
 	if (!input_method_keyboard_grab_forward_modifiers(keyboard)) {
+		/* Send modifiers to focused client */
 		wlr_seat_keyboard_notify_modifiers(seat->seat,
 			&wlr_keyboard->modifiers);
+
+		/*
+		 * Also broadcast them to non-keyboard-focused clients.
+		 *
+		 * The Wayland protocol does not specify that modifiers are
+		 * broadcast, so this is not something clients can rely on in
+		 * other compositors.
+		 *
+		 * Sway used to broadcast modifiers but stopped doing so to
+		 * avoid waking up all clients when the modifiers change.
+		 *
+		 * By testing with foot and Ctrl+scroll to change font size, it
+		 * appears that Mutter does not pass modifiers to unfocused
+		 * clients, whereas KWin and Weston pass modifiers to clients
+		 * with pointer-focus.
+		 *
+		 * This could be made configurable if there are unintended
+		 * consequences. If so, modifiers ought to still be passed to
+		 * clients with pointer-focus (see issue #2271)
+		 */
+		broadcast_modifiers_to_unfocused_clients(seat->seat,
+			&wlr_keyboard->modifiers);
 	}
 }
 
@@ -303,7 +364,8 @@ get_keyinfo(struct wlr_keyboard *wlr_keyboard, uint32_t evdev_keycode)
 }
 
 static bool
-handle_key_release(struct server *server, uint32_t evdev_keycode)
+handle_key_release(struct server *server, uint32_t evdev_keycode,
+		bool is_modifier_key)
 {
 	/*
 	 * Release events for keys that were not bound should always be
@@ -326,11 +388,33 @@ handle_key_release(struct server *server, uint32_t evdev_keycode)
 		end_cycling(server);
 	}
 
+	key_state_bound_key_remove(evdev_keycode);
+
+	/*
+	 * There are some clients (for example blender) that want to see the
+	 * modifier-release-event even if it was part of a keybinds. This is
+	 * treated as a special case and can only be achieved by configuration.
+	 *
+	 * Most clients (including those using Qt and GTK) are setup to not see
+	 * these modifier release events - and actually misbehave if they do.
+	 * For example Firefox shows the menu bar if alt is pressed and then
+	 * released, whereas if only pressed (because the release is absorbed)
+	 * nothing happens. So, if Firefox saw bound modifier-release-events it
+	 * would show the menu bar every time the window-switcher is used with
+	 * alt-tab.
+	 */
+	struct view *view = server->active_view;
+	if (is_modifier_key && view) {
+		if (window_rules_get_property(view, "wantAbsorbedModifierReleaseEvents")
+				== LAB_PROP_TRUE) {
+			return false;
+		}
+	}
+
 	/*
 	 * If a press event was handled by a compositor binding, then do
 	 * not forward the corresponding release event to clients.
 	 */
-	key_state_bound_key_remove(evdev_keycode);
 	return true;
 }
 
@@ -465,7 +549,8 @@ handle_compositor_keybindings(struct keyboard *keyboard,
 			actions_run(NULL, server, &cur_keybind->actions, NULL);
 			return true;
 		} else {
-			return handle_key_release(server, event->keycode);
+			return handle_key_release(server, event->keycode,
+				keyinfo.is_modifier);
 		}
 	}
 
