@@ -1,4 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only
+#include <assert.h>
 #include "edges.h"
 #include "input/keyboard.h"
 #include "labwc.h"
@@ -8,19 +9,49 @@
 #include "view.h"
 #include "window-rules.h"
 
+/*
+ *   pos_old  pos_cursor
+ *      v         v
+ *      +---------+-------------------+
+ *      <-----------size_old---------->
+ *
+ *      return value
+ *           v
+ *           +----+---------+
+ *           <---size_new--->
+ */
 static int
-max_move_scale(double pos_cursor, double pos_current,
-	double size_current, double size_orig)
+max_move_scale(double pos_cursor, double pos_old, double size_old,
+		double size_new)
 {
-	double anchor_frac = (pos_cursor - pos_current) / size_current;
-	int pos_new = pos_cursor - (size_orig * anchor_frac);
-	if (pos_new < pos_current) {
+	double anchor_frac = (pos_cursor - pos_old) / size_old;
+	int pos_new = pos_cursor - (size_new * anchor_frac);
+	if (pos_new < pos_old) {
 		/* Clamp by using the old offsets of the maximized window */
-		pos_new = pos_current;
+		pos_new = pos_old;
 	}
 	return pos_new;
 }
 
+void
+interactive_anchor_to_cursor(struct server *server, struct wlr_box *geo)
+{
+	assert(server->input_mode == LAB_INPUT_STATE_MOVE);
+	if (wlr_box_empty(geo)) {
+		return;
+	}
+	/* Resize grab_box while anchoring it to grab_box.{x,y} */
+	server->grab_box.x = max_move_scale(server->grab_x, server->grab_box.x,
+		server->grab_box.width, geo->width);
+	server->grab_box.y = max_move_scale(server->grab_y, server->grab_box.y,
+		server->grab_box.height, geo->height);
+	server->grab_box.width = geo->width;
+	server->grab_box.height = geo->height;
+
+	geo->x = server->grab_box.x + (server->seat.cursor->x - server->grab_x);
+	geo->y = server->grab_box.y + (server->seat.cursor->y - server->grab_y);
+}
+
 void
 interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 {
@@ -31,7 +62,6 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 	 */
 	struct server *server = view->server;
 	struct seat *seat = &server->seat;
-	struct wlr_box geometry = view->current;
 
 	if (server->input_mode != LAB_INPUT_STATE_PASSTHROUGH) {
 		return;
@@ -55,25 +85,8 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 			 */
 			return;
 		}
-		if (!view_is_floating(view)) {
-			/*
-			 * Un-maximize, unshade and restore natural
-			 * width/height.
-			 * Don't reset tiled state yet since we may want
-			 * to keep it (in the snap-to-maximize case).
-			 */
-			geometry = view->natural_geometry;
-			geometry.x = max_move_scale(seat->cursor->x,
-				view->current.x, view->current.width,
-				geometry.width);
-			geometry.y = max_move_scale(seat->cursor->y,
-				view->current.y, view->current.height,
-				geometry.height);
 
-			view_set_shade(view, false);
-			view_set_untiled(view);
-			view_restore_to(view, geometry);
-		} else {
+		if (view_is_floating(view)) {
 			/* Store natural geometry at start of move */
 			view_store_natural_geometry(view);
 			view_invalidate_last_layout_geometry(view);
@@ -103,7 +116,7 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 
 		/*
 		 * If tiled or maximized in only one direction, reset
-		 * tiled/maximized state but keep the same geometry as
+		 * maximized/tiled state but keep the same geometry as
 		 * the starting point for the resize.
 		 */
 		view_set_untiled(view);
@@ -120,8 +133,26 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 	/* Remember view and cursor positions at start of move/resize */
 	server->grab_x = seat->cursor->x;
 	server->grab_y = seat->cursor->y;
-	server->grab_box = geometry;
+	server->grab_box = view->current;
 	server->resize_edges = edges;
+
+	/*
+	 * Un-tile maximized/tiled view immediately if <unSnapThreshold> is
+	 * zero. Otherwise, un-tile it later in cursor motion handler.
+	 * If the natural geometry is unknown (possible with xdg-shell views),
+	 * then we set a size of 0x0 here and determine the correct geometry
+	 * later. See do_late_positioning() in xdg.c.
+	 */
+	if (mode == LAB_INPUT_STATE_MOVE && !view_is_floating(view)
+			&& rc.unsnap_threshold <= 0) {
+		struct wlr_box natural_geo = view->natural_geometry;
+		interactive_anchor_to_cursor(server, &natural_geo);
+		/* Shaded clients will not process resize events until unshaded */
+		view_set_shade(view, false);
+		view_set_untiled(view);
+		view_restore_to(view, natural_geo);
+	}
+
 	if (rc.resize_indicator) {
 		resize_indicator_show(view);
 	}
@@ -133,6 +164,10 @@ interactive_begin(struct view *view, enum input_mode mode, uint32_t edges)
 enum view_edge
 edge_from_cursor(struct seat *seat, struct output **dest_output)
 {
+	if (!view_is_floating(seat->server->grabbed_view)) {
+		return VIEW_EDGE_INVALID;
+	}
+
 	int snap_range = rc.snap_edge_range;
 	if (!snap_range) {
 		return VIEW_EDGE_INVALID;
