@@ -10,7 +10,7 @@
 #include "common/scene-helpers.h"
 #include "common/string-helpers.h"
 #if HAVE_LIBSFDO
-#include "icon-loader.h"
+#include "desktop-entry.h"
 #endif
 #include "labwc.h"
 #include "node.h"
@@ -38,6 +38,7 @@ ssd_titlebar_create(struct ssd *ssd)
 	struct wlr_scene_tree *parent;
 	struct wlr_buffer *corner_top_left;
 	struct wlr_buffer *corner_top_right;
+	int active;
 
 	ssd->titlebar.tree = wlr_scene_tree_create(ssd->tree);
 
@@ -45,23 +46,18 @@ ssd_titlebar_create(struct ssd *ssd)
 	FOR_EACH_STATE(ssd, subtree) {
 		subtree->tree = wlr_scene_tree_create(ssd->titlebar.tree);
 		parent = subtree->tree;
-		wlr_scene_node_set_position(&parent->node, 0, -theme->title_height);
-		if (subtree == &ssd->titlebar.active) {
-			color = theme->window_active_title_bg_color;
-			corner_top_left = &theme->corner_top_left_active_normal->base;
-			corner_top_right = &theme->corner_top_right_active_normal->base;
-		} else {
-			color = theme->window_inactive_title_bg_color;
-			corner_top_left = &theme->corner_top_left_inactive_normal->base;
-			corner_top_right = &theme->corner_top_right_inactive_normal->base;
-
-			wlr_scene_node_set_enabled(&parent->node, false);
-		}
+		active = (subtree == &ssd->titlebar.active) ?
+			THEME_ACTIVE : THEME_INACTIVE;
+		color = theme->window[active].title_bg_color;
+		corner_top_left = &theme->window[active].corner_top_left_normal->base;
+		corner_top_right = &theme->window[active].corner_top_right_normal->base;
+		wlr_scene_node_set_enabled(&parent->node, active);
+		wlr_scene_node_set_position(&parent->node, 0, -theme->titlebar_height);
 		wl_list_init(&subtree->parts);
 
 		/* Background */
 		add_scene_rect(&subtree->parts, LAB_SSD_PART_TITLEBAR, parent,
-			width - corner_width * 2, theme->title_height,
+			width - corner_width * 2, theme->titlebar_height,
 			corner_width, 0, color);
 		add_scene_buffer(&subtree->parts, LAB_SSD_PART_TITLEBAR_CORNER_LEFT, parent,
 			corner_top_left, -rc.theme->border_width, -rc.theme->border_width);
@@ -69,15 +65,12 @@ ssd_titlebar_create(struct ssd *ssd)
 			corner_top_right, width - corner_width,
 			-rc.theme->border_width);
 
-		int active = (subtree == &ssd->titlebar.active) ?
-				THEME_ACTIVE : THEME_INACTIVE;
-
 		/* Buttons */
 		struct title_button *b;
 		int x = theme->window_titlebar_padding_width;
 
 		/* Center vertically within titlebar */
-		int y = (theme->title_height - theme->window_button_height) / 2;
+		int y = (theme->titlebar_height - theme->window_button_height) / 2;
 
 		wl_list_for_each(b, &rc.title_buttons_left, link) {
 			struct lab_data_buffer **buffers =
@@ -102,14 +95,16 @@ ssd_titlebar_create(struct ssd *ssd)
 	ssd_update_title(ssd);
 	ssd_update_window_icon(ssd);
 
-	set_squared_corners(ssd, false);
-
 	bool maximized = view->maximized == VIEW_AXIS_BOTH;
+	bool squared = ssd_should_be_squared(ssd);
 	if (maximized) {
-		set_squared_corners(ssd, true);
 		set_alt_button_icon(ssd, LAB_SSD_BUTTON_MAXIMIZE, true);
 		ssd->state.was_maximized = true;
 	}
+	if (squared) {
+		ssd->state.was_squared = true;
+	}
+	set_squared_corners(ssd, maximized || squared);
 
 	if (view->shaded) {
 		set_alt_button_icon(ssd, LAB_SSD_BUTTON_SHADE, true);
@@ -118,11 +113,6 @@ ssd_titlebar_create(struct ssd *ssd)
 	if (view->visible_on_all_workspaces) {
 		set_alt_button_icon(ssd, LAB_SSD_BUTTON_OMNIPRESENT, true);
 	}
-
-	if (ssd_should_be_squared(ssd)) {
-		set_squared_corners(ssd, true);
-		ssd->state.was_squared = true;
-	}
 }
 
 static void
@@ -159,8 +149,8 @@ set_squared_corners(struct ssd *ssd, bool enable)
 	FOR_EACH_STATE(ssd, subtree) {
 		part = ssd_get_part(&subtree->parts, LAB_SSD_PART_TITLEBAR);
 		wlr_scene_node_set_position(part->node, x, 0);
-		wlr_scene_rect_set_size(
-			wlr_scene_rect_from_node(part->node), width - 2 * x, theme->title_height);
+		wlr_scene_rect_set_size(wlr_scene_rect_from_node(part->node),
+			width - 2 * x, theme->titlebar_height);
 
 		part = ssd_get_part(&subtree->parts, LAB_SSD_PART_TITLEBAR_CORNER_LEFT);
 		wlr_scene_node_set_enabled(part->node, !enable);
@@ -298,7 +288,7 @@ ssd_titlebar_update(struct ssd *ssd)
 	update_visible_buttons(ssd);
 
 	/* Center buttons vertically within titlebar */
-	int y = (theme->title_height - theme->window_button_height) / 2;
+	int y = (theme->titlebar_height - theme->window_button_height) / 2;
 	int x;
 	struct ssd_part *part;
 	struct ssd_sub_tree *subtree;
@@ -308,7 +298,7 @@ ssd_titlebar_update(struct ssd *ssd)
 		part = ssd_get_part(&subtree->parts, LAB_SSD_PART_TITLEBAR);
 		wlr_scene_rect_set_size(
 			wlr_scene_rect_from_node(part->node),
-			width - bg_offset * 2, theme->title_height);
+			width - bg_offset * 2, theme->titlebar_height);
 
 		x = theme->window_titlebar_padding_width;
 		wl_list_for_each(b, &rc.title_buttons_left, link) {
@@ -393,7 +383,7 @@ ssd_update_title_positions(struct ssd *ssd, int offset_left, int offset_right)
 		buffer_width = part->buffer ? part->buffer->width : 0;
 		buffer_height = part->buffer ? part->buffer->height : 0;
 		x = offset_left;
-		y = (theme->title_height - buffer_height) / 2;
+		y = (theme->titlebar_height - buffer_height) / 2;
 
 		if (title_bg_width <= 0) {
 			wlr_scene_node_set_enabled(part->node, false);
@@ -474,23 +464,19 @@ ssd_update_title(struct ssd *ssd)
 	struct ssd_part *part;
 	struct ssd_sub_tree *subtree;
 	struct ssd_state_title_width *dstate;
+	int active;
 
 	int offset_left, offset_right;
 	get_title_offsets(ssd, &offset_left, &offset_right);
 	int title_bg_width = view->current.width - offset_left - offset_right;
 
 	FOR_EACH_STATE(ssd, subtree) {
-		if (subtree == &ssd->titlebar.active) {
-			dstate = &state->active;
-			text_color = theme->window_active_label_text_color;
-			bg_color = theme->window_active_title_bg_color;
-			font = &rc.font_activewindow;
-		} else {
-			dstate = &state->inactive;
-			text_color = theme->window_inactive_label_text_color;
-			bg_color = theme->window_inactive_title_bg_color;
-			font = &rc.font_inactivewindow;
-		}
+		active = (subtree == &ssd->titlebar.active) ?
+			THEME_ACTIVE : THEME_INACTIVE;
+		dstate = active ? &state->active : &state->inactive;
+		text_color = theme->window[active].label_text_color;
+		bg_color = theme->window[active].title_bg_color;
+		font = &rc.font_activewindow;
 
 		if (title_bg_width <= 0) {
 			dstate->truncated = true;
@@ -621,7 +607,7 @@ ssd_update_window_icon(struct ssd *ssd)
 	 */
 	float icon_scale = output_max_scale(ssd->view->server);
 
-	struct lab_data_buffer *icon_buffer = icon_loader_lookup(
+	struct lab_data_buffer *icon_buffer = desktop_entry_icon_lookup(
 		ssd->view->server, app_id, icon_size, icon_scale);
 	if (!icon_buffer) {
 		wlr_log(WLR_DEBUG, "icon could not be loaded for %s", app_id);
