@@ -80,15 +80,19 @@ zdrop(struct lab_data_buffer **buffer)
 	}
 }
 
-static struct lab_data_buffer *
+struct icon_drawing_context {
+	struct lab_data_buffer *buffer;
+	cairo_t *cairo;
+};
+
+static struct icon_drawing_context
 copy_icon_buffer(struct theme *theme, struct lab_data_buffer *icon_buffer)
 {
 	assert(icon_buffer);
 
-	struct surface_context icon =
-		get_cairo_surface_from_lab_data_buffer(icon_buffer);
-	int icon_width = cairo_image_surface_get_width(icon.surface);
-	int icon_height = cairo_image_surface_get_height(icon.surface);
+	cairo_surface_t *surface = icon_buffer->surface;
+	int icon_width = cairo_image_surface_get_width(surface);
+	int icon_height = cairo_image_surface_get_height(surface);
 
 	int width = theme->window_button_width;
 	int height = theme->window_button_height;
@@ -111,9 +115,9 @@ copy_icon_buffer(struct theme *theme, struct lab_data_buffer *icon_buffer)
 	int buffer_height = (double)height * scale;
 	struct lab_data_buffer *buffer = buffer_create_cairo(
 		buffer_width, buffer_height, 1.0);
-	cairo_t *cairo = buffer->cairo;
+	cairo_t *cairo = cairo_create(buffer->surface);
 
-	cairo_set_source_surface(cairo, icon.surface,
+	cairo_set_source_surface(cairo, surface,
 		(buffer_width - icon_width) / 2, (buffer_height - icon_height) / 2);
 	cairo_paint(cairo);
 
@@ -123,11 +127,10 @@ copy_icon_buffer(struct theme *theme, struct lab_data_buffer *icon_buffer)
 	 */
 	cairo_scale(cairo, scale, scale);
 
-	if (icon.is_duplicate) {
-		cairo_surface_destroy(icon.surface);
-	}
-
-	return buffer;
+	return (struct icon_drawing_context){
+		.buffer = buffer,
+		.cairo = cairo,
+	};
 }
 
 static void
@@ -141,8 +144,9 @@ create_hover_fallback(struct theme *theme,
 	int width = theme->window_button_width;
 	int height = theme->window_button_height;
 
-	*hover_buffer = copy_icon_buffer(theme, icon_buffer);
-	cairo_t *cairo = (*hover_buffer)->cairo;
+	struct icon_drawing_context ctx = copy_icon_buffer(theme, icon_buffer);
+	*hover_buffer = ctx.buffer;
+	cairo_t *cairo = ctx.cairo;
 
 	/* Overlay (pre-multiplied alpha) */
 	float overlay_color[4] = { 0.15f, 0.15f, 0.15f, 0.3f};
@@ -161,6 +165,7 @@ create_hover_fallback(struct theme *theme,
 	cairo_fill(cairo);
 
 	cairo_surface_flush(cairo_get_target(cairo));
+	cairo_destroy(cairo);
 }
 
 static void
@@ -168,8 +173,9 @@ create_rounded_buffer(struct theme *theme, enum corner corner,
 		struct lab_data_buffer **rounded_buffer,
 		struct lab_data_buffer *icon_buffer)
 {
-	*rounded_buffer = copy_icon_buffer(theme, icon_buffer);
-	cairo_t *cairo = (*rounded_buffer)->cairo;
+	struct icon_drawing_context ctx = copy_icon_buffer(theme, icon_buffer);
+	*rounded_buffer = ctx.buffer;
+	cairo_t *cairo = ctx.cairo;
 
 	int width = theme->window_button_width;
 	int height = theme->window_button_height;
@@ -179,7 +185,7 @@ create_rounded_buffer(struct theme *theme, enum corner corner,
 	 * border. See the picture in #2189 for reference.
 	 */
 	int margin_x = theme->window_titlebar_padding_width;
-	int margin_y = (theme->title_height - theme->window_button_height) / 2;
+	int margin_y = (theme->titlebar_height - theme->window_button_height) / 2;
 	float white[4] = {1, 1, 1, 1};
 	struct rounded_corner_ctx rounded_ctx = {
 		.box = &(struct wlr_box){
@@ -194,12 +200,13 @@ create_rounded_buffer(struct theme *theme, enum corner corner,
 	};
 	struct lab_data_buffer *mask_buffer = rounded_rect(&rounded_ctx);
 	cairo_set_operator(cairo, CAIRO_OPERATOR_DEST_IN);
-	cairo_set_source_surface(cairo, cairo_get_target(mask_buffer->cairo),
+	cairo_set_source_surface(cairo, mask_buffer->surface,
 		(corner == LAB_CORNER_TOP_LEFT) ? -margin_x : 0,
 		-margin_y);
 	cairo_paint(cairo);
 
 	cairo_surface_flush(cairo_get_target(cairo));
+	cairo_destroy(cairo);
 	wlr_buffer_drop(&mask_buffer->base);
 }
 
@@ -562,22 +569,18 @@ theme_builtin(struct theme *theme, struct server *server)
 	theme->border_width = 1;
 	theme->window_titlebar_padding_height = 0;
 	theme->window_titlebar_padding_width = 0;
-	theme->title_height = INT_MIN;
-	theme->menu_overlap_x = 0;
-	theme->menu_overlap_y = 0;
 
-	parse_hexstr("#e1dedb", theme->window_active_border_color);
-	parse_hexstr("#f6f5f4", theme->window_inactive_border_color);
+	parse_hexstr("#aaaaaa", theme->window[THEME_ACTIVE].border_color);
+	parse_hexstr("#aaaaaa", theme->window[THEME_INACTIVE].border_color);
 
 	parse_hexstr("#ff0000", theme->window_toggled_keybinds_color);
 
-	parse_hexstr("#e1dedb", theme->window_active_title_bg_color);
-	parse_hexstr("#f6f5f4", theme->window_inactive_title_bg_color);
+	parse_hexstr("#e1dedb", theme->window[THEME_ACTIVE].title_bg_color);
+	parse_hexstr("#f6f5f4", theme->window[THEME_INACTIVE].title_bg_color);
 
-	parse_hexstr("#000000", theme->window_active_label_text_color);
-	parse_hexstr("#000000", theme->window_inactive_label_text_color);
+	parse_hexstr("#000000", theme->window[THEME_ACTIVE].label_text_color);
+	parse_hexstr("#000000", theme->window[THEME_INACTIVE].label_text_color);
 	theme->window_label_text_justify = parse_justification("Center");
-	theme->menu_title_text_justify = parse_justification("Center");
 
 	theme->window_button_width = 26;
 	theme->window_button_height = 26;
@@ -592,29 +595,32 @@ theme_builtin(struct theme *theme, struct server *server)
 			theme->window[THEME_ACTIVE].button_colors[type]);
 	}
 
-	theme->window_active_shadow_size = 60;
-	theme->window_inactive_shadow_size = 40;
-	parse_hexstr("#00000060", theme->window_active_shadow_color);
-	parse_hexstr("#00000040", theme->window_inactive_shadow_color);
+	theme->window[THEME_ACTIVE].shadow_size = 60;
+	theme->window[THEME_INACTIVE].shadow_size = 40;
+	parse_hexstr("#00000060", theme->window[THEME_ACTIVE].shadow_color);
+	parse_hexstr("#00000040", theme->window[THEME_INACTIVE].shadow_color);
 
+	theme->menu_overlap_x = 0;
+	theme->menu_overlap_y = 0;
+	theme->menu_min_width = 20;
+	theme->menu_max_width = 200;
+	theme->menu_border_width = INT_MIN;
+	theme->menu_border_color[0] = FLT_MIN;
+
+	theme->menu_items_padding_x = 7;
+	theme->menu_items_padding_y = 4;
 	parse_hexstr("#fcfbfa", theme->menu_items_bg_color);
 	parse_hexstr("#000000", theme->menu_items_text_color);
 	parse_hexstr("#e1dedb", theme->menu_items_active_bg_color);
 	parse_hexstr("#000000", theme->menu_items_active_text_color);
 
-	theme->menu_item_padding_x = 7;
-	theme->menu_item_padding_y = 4;
-
-	theme->menu_min_width = 20;
-	theme->menu_max_width = 200;
-
 	theme->menu_separator_line_thickness = 1;
 	theme->menu_separator_padding_width = 6;
 	theme->menu_separator_padding_height = 3;
 	parse_hexstr("#888888", theme->menu_separator_color);
 
 	parse_hexstr("#589bda", theme->menu_title_bg_color);
-
+	theme->menu_title_text_justify = parse_justification("Center");
 	parse_hexstr("#ffffff", theme->menu_title_text_color);
 
 	theme->osd_window_switcher_width = 600;
@@ -708,36 +714,17 @@ entry(struct theme *theme, const char *key, const char *value)
 	if (match_glob(key, "padding.height")) {
 		wlr_log(WLR_ERROR, "padding.height is no longer supported");
 	}
-	if (match_glob(key, "menu.items.padding.x")) {
-		theme->menu_item_padding_x = get_int_if_positive(
-			value, "menu.items.padding.x");
-	}
-	if (match_glob(key, "menu.items.padding.y")) {
-		theme->menu_item_padding_y = get_int_if_positive(
-			value, "menu.items.padding.y");
-	}
-	if (match_glob(key, "menu.title.text.justify")) {
-		theme->menu_title_text_justify = parse_justification(value);
-	}
-	if (match_glob(key, "menu.overlap.x")) {
-		theme->menu_overlap_x = get_int_if_positive(
-			value, "menu.overlap.x");
-	}
-	if (match_glob(key, "menu.overlap.y")) {
-		theme->menu_overlap_y = get_int_if_positive(
-			value, "menu.overlap.y");
-	}
 
 	if (match_glob(key, "window.active.border.color")) {
-		parse_hexstr(value, theme->window_active_border_color);
+		parse_hexstr(value, theme->window[THEME_ACTIVE].border_color);
 	}
 	if (match_glob(key, "window.inactive.border.color")) {
-		parse_hexstr(value, theme->window_inactive_border_color);
+		parse_hexstr(value, theme->window[THEME_INACTIVE].border_color);
 	}
 	/* border.color is obsolete, but handled for backward compatibility */
 	if (match_glob(key, "border.color")) {
-		parse_hexstr(value, theme->window_active_border_color);
-		parse_hexstr(value, theme->window_inactive_border_color);
+		parse_hexstr(value, theme->window[THEME_ACTIVE].border_color);
+		parse_hexstr(value, theme->window[THEME_INACTIVE].border_color);
 	}
 
 	if (match_glob(key, "window.active.indicator.toggled-keybind.color")) {
@@ -745,17 +732,17 @@ entry(struct theme *theme, const char *key, const char *value)
 	}
 
 	if (match_glob(key, "window.active.title.bg.color")) {
-		parse_hexstr(value, theme->window_active_title_bg_color);
+		parse_hexstr(value, theme->window[THEME_ACTIVE].title_bg_color);
 	}
 	if (match_glob(key, "window.inactive.title.bg.color")) {
-		parse_hexstr(value, theme->window_inactive_title_bg_color);
+		parse_hexstr(value, theme->window[THEME_INACTIVE].title_bg_color);
 	}
 
 	if (match_glob(key, "window.active.label.text.color")) {
-		parse_hexstr(value, theme->window_active_label_text_color);
+		parse_hexstr(value, theme->window[THEME_ACTIVE].label_text_color);
 	}
 	if (match_glob(key, "window.inactive.label.text.color")) {
-		parse_hexstr(value, theme->window_inactive_label_text_color);
+		parse_hexstr(value, theme->window[THEME_INACTIVE].label_text_color);
 	}
 	if (match_glob(key, "window.label.text.justify")) {
 		theme->window_label_text_justify = parse_justification(value);
@@ -858,20 +845,26 @@ entry(struct theme *theme, const char *key, const char *value)
 
 	/* window drop-shadows */
 	if (match_glob(key, "window.active.shadow.size")) {
-		theme->window_active_shadow_size = get_int_if_positive(
+		theme->window[THEME_ACTIVE].shadow_size = get_int_if_positive(
 			value, "window.active.shadow.size");
 	}
 	if (match_glob(key, "window.inactive.shadow.size")) {
-		theme->window_inactive_shadow_size = get_int_if_positive(
+		theme->window[THEME_INACTIVE].shadow_size = get_int_if_positive(
 			value, "window.inactive.shadow.size");
 	}
 	if (match_glob(key, "window.active.shadow.color")) {
-		parse_hexstr(value, theme->window_active_shadow_color);
+		parse_hexstr(value, theme->window[THEME_ACTIVE].shadow_color);
 	}
 	if (match_glob(key, "window.inactive.shadow.color")) {
-		parse_hexstr(value, theme->window_inactive_shadow_color);
+		parse_hexstr(value, theme->window[THEME_INACTIVE].shadow_color);
 	}
 
+	if (match_glob(key, "menu.overlap.x")) {
+		theme->menu_overlap_x = atoi(value);
+	}
+	if (match_glob(key, "menu.overlap.y")) {
+		theme->menu_overlap_y = atoi(value);
+	}
 	if (match_glob(key, "menu.width.min")) {
 		theme->menu_min_width = get_int_if_positive(
 			value, "menu.width.min");
@@ -880,7 +873,22 @@ entry(struct theme *theme, const char *key, const char *value)
 		theme->menu_max_width = get_int_if_positive(
 			value, "menu.width.max");
 	}
+	if (match_glob(key, "menu.border.width")) {
+		theme->menu_border_width = get_int_if_positive(
+			value, "menu.border.width");
+	}
+	if (match_glob(key, "menu.border.color")) {
+		parse_hexstr(value, theme->menu_border_color);
+	}
 
+	if (match_glob(key, "menu.items.padding.x")) {
+		theme->menu_items_padding_x = get_int_if_positive(
+			value, "menu.items.padding.x");
+	}
+	if (match_glob(key, "menu.items.padding.y")) {
+		theme->menu_items_padding_y = get_int_if_positive(
+			value, "menu.items.padding.y");
+	}
 	if (match_glob(key, "menu.items.bg.color")) {
 		parse_hexstr(value, theme->menu_items_bg_color);
 	}
@@ -913,7 +921,9 @@ entry(struct theme *theme, const char *key, const char *value)
 	if (match_glob(key, "menu.title.bg.color")) {
 		parse_hexstr(value, theme->menu_title_bg_color);
 	}
-
+	if (match_glob(key, "menu.title.text.justify")) {
+		theme->menu_title_text_justify = parse_justification(value);
+	}
 	if (match_glob(key, "menu.title.text.color")) {
 		parse_hexstr(value, theme->menu_title_text_color);
 	}
@@ -1090,8 +1100,8 @@ rounded_rect(struct rounded_corner_ctx *ctx)
 	/* TODO: scale */
 	buffer = buffer_create_cairo(w, h, 1);
 
-	cairo_t *cairo = buffer->cairo;
-	cairo_surface_t *surf = cairo_get_target(cairo);
+	cairo_surface_t *surf = buffer->surface;
+	cairo_t *cairo = cairo_create(surf);
 
 	/* set transparent background */
 	cairo_set_operator(cairo, CAIRO_OPERATOR_CLEAR);
@@ -1233,6 +1243,7 @@ rounded_rect(struct rounded_corner_ctx *ctx)
 
 out:
 	cairo_surface_flush(surf);
+	cairo_destroy(cairo);
 
 	return buffer;
 }
@@ -1246,31 +1257,22 @@ create_corners(struct theme *theme)
 		.x = 0,
 		.y = 0,
 		.width = corner_width + theme->border_width,
-		.height = theme->title_height + theme->border_width,
+		.height = theme->titlebar_height + theme->border_width,
 	};
 
-	struct rounded_corner_ctx ctx = {
-		.box = &box,
-		.radius = rc.corner_radius,
-		.line_width = theme->border_width,
-		.fill_color = theme->window_active_title_bg_color,
-		.border_color = theme->window_active_border_color,
-		.corner = LAB_CORNER_TOP_LEFT,
-	};
-	theme->corner_top_left_active_normal = rounded_rect(&ctx);
-
-	ctx.fill_color = theme->window_inactive_title_bg_color,
-	ctx.border_color = theme->window_inactive_border_color,
-	theme->corner_top_left_inactive_normal = rounded_rect(&ctx);
-
-	ctx.corner = LAB_CORNER_TOP_RIGHT;
-	ctx.fill_color = theme->window_active_title_bg_color,
-	ctx.border_color = theme->window_active_border_color,
-	theme->corner_top_right_active_normal = rounded_rect(&ctx);
-
-	ctx.fill_color = theme->window_inactive_title_bg_color,
-	ctx.border_color = theme->window_inactive_border_color,
-	theme->corner_top_right_inactive_normal = rounded_rect(&ctx);
+	for (int active = THEME_INACTIVE; active <= THEME_ACTIVE; active++) {
+		struct rounded_corner_ctx ctx = {
+			.box = &box,
+			.radius = rc.corner_radius,
+			.line_width = theme->border_width,
+			.fill_color = theme->window[active].title_bg_color,
+			.border_color = theme->window[active].border_color,
+			.corner = LAB_CORNER_TOP_LEFT,
+		};
+		theme->window[active].corner_top_left_normal = rounded_rect(&ctx);
+		ctx.corner = LAB_CORNER_TOP_RIGHT;
+		theme->window[active].corner_top_right_normal = rounded_rect(&ctx);
+	}
 }
 
 /*
@@ -1388,68 +1390,50 @@ shadow_corner_gradient(struct lab_data_buffer *buffer, int visible_size,
 }
 
 static void
-create_shadows(struct theme *theme)
+create_shadow(struct theme *theme, int active)
 {
 	/* Size of shadow visible extending beyond the window */
-	int visible_active_size = theme->window_active_shadow_size;
-	int visible_inactive_size = theme->window_inactive_shadow_size;
+	int visible_size = theme->window[active].shadow_size;
 	/* How far inside the window the shadow inset begins */
-	int inset_active = (double)visible_active_size * SSD_SHADOW_INSET;
-	int inset_inactive = (double)visible_inactive_size * SSD_SHADOW_INSET;
+	int inset = (double)visible_size * SSD_SHADOW_INSET;
 	/* Total width including visible and obscured portion */
-	int total_active_size = visible_active_size + inset_active;
-	int total_inactive_size = visible_inactive_size + inset_inactive;
+	int total_size = visible_size + inset;
 
 	/*
 	 * Edge shadows don't need to be inset so the buffers are sized just for
 	 * the visible width.  Corners are inset so the buffers are larger for
 	 * this.
 	 */
-	if (visible_active_size > 0) {
-		theme->shadow_edge_active = buffer_create_cairo(
-			visible_active_size, 1, 1.0);
-		theme->shadow_corner_top_active = buffer_create_cairo(
-			total_active_size, total_active_size, 1.0);
-		theme->shadow_corner_bottom_active = buffer_create_cairo(
-			total_active_size, total_active_size, 1.0);
-		if (!theme->shadow_corner_top_active
-				|| !theme->shadow_corner_bottom_active
-				|| !theme->shadow_edge_active) {
-			wlr_log(WLR_ERROR, "Failed to allocate shadow buffer");
-			return;
-		}
-	}
-	if (visible_inactive_size > 0) {
-		theme->shadow_edge_inactive = buffer_create_cairo(
-			visible_inactive_size, 1, 1.0);
-		theme->shadow_corner_top_inactive = buffer_create_cairo(
-			total_inactive_size, total_inactive_size, 1.0);
-		theme->shadow_corner_bottom_inactive = buffer_create_cairo(
-			total_inactive_size, total_inactive_size, 1.0);
-		if (!theme->shadow_corner_top_inactive
-				|| !theme->shadow_corner_bottom_inactive
-				|| !theme->shadow_edge_inactive) {
+	if (visible_size > 0) {
+		theme->window[active].shadow_edge = buffer_create_cairo(
+			visible_size, 1, 1.0);
+		theme->window[active].shadow_corner_top = buffer_create_cairo(
+			total_size, total_size, 1.0);
+		theme->window[active].shadow_corner_bottom = buffer_create_cairo(
+			total_size, total_size, 1.0);
+		if (!theme->window[active].shadow_corner_top
+				|| !theme->window[active].shadow_corner_bottom
+				|| !theme->window[active].shadow_edge) {
 			wlr_log(WLR_ERROR, "Failed to allocate shadow buffer");
 			return;
 		}
 	}
 
-	shadow_edge_gradient(theme->shadow_edge_active, visible_active_size,
-		total_active_size, theme->window_active_shadow_color);
-	shadow_edge_gradient(theme->shadow_edge_inactive, visible_inactive_size,
-		total_inactive_size, theme->window_inactive_shadow_color);
-	shadow_corner_gradient(theme->shadow_corner_top_active,
-		visible_active_size, total_active_size,
-		theme->title_height, theme->window_active_shadow_color);
-	shadow_corner_gradient(theme->shadow_corner_bottom_active,
-		visible_active_size, total_active_size, 0,
-		theme->window_active_shadow_color);
-	shadow_corner_gradient(theme->shadow_corner_top_inactive,
-		visible_inactive_size, total_inactive_size,
-		theme->title_height, theme->window_inactive_shadow_color);
-	shadow_corner_gradient(theme->shadow_corner_bottom_inactive,
-		visible_inactive_size, total_inactive_size, 0,
-		theme->window_inactive_shadow_color);
+	shadow_edge_gradient(theme->window[active].shadow_edge, visible_size,
+		total_size, theme->window[active].shadow_color);
+	shadow_corner_gradient(theme->window[active].shadow_corner_top,
+		visible_size, total_size,
+		theme->titlebar_height, theme->window[active].shadow_color);
+	shadow_corner_gradient(theme->window[active].shadow_corner_bottom,
+		visible_size, total_size, 0,
+		theme->window[active].shadow_color);
+}
+
+static void
+create_shadows(struct theme *theme)
+{
+	create_shadow(theme, THEME_INACTIVE);
+	create_shadow(theme, THEME_ACTIVE);
 }
 
 static void
@@ -1475,17 +1459,20 @@ get_titlebar_height(struct theme *theme)
 static void
 post_processing(struct theme *theme)
 {
-	theme->title_height = get_titlebar_height(theme);
+	theme->titlebar_height = get_titlebar_height(theme);
 
 	theme->menu_item_height = font_height(&rc.font_menuitem)
-		+ 2 * theme->menu_item_padding_y;
+		+ 2 * theme->menu_items_padding_y;
+
+	theme->menu_header_height = font_height(&rc.font_menuheader)
+		+ 2 * theme->menu_items_padding_y;
 
 	theme->osd_window_switcher_item_height = font_height(&rc.font_osd)
 		+ 2 * theme->osd_window_switcher_item_padding_y
 		+ 2 * theme->osd_window_switcher_item_active_border_width;
 
-	if (rc.corner_radius >= theme->title_height) {
-		rc.corner_radius = theme->title_height - 1;
+	if (rc.corner_radius >= theme->titlebar_height) {
+		rc.corner_radius = theme->titlebar_height - 1;
 	}
 
 	int min_button_hover_radius =
@@ -1501,10 +1488,19 @@ post_processing(struct theme *theme)
 		theme->menu_max_width = theme->menu_min_width;
 	}
 
+	if (theme->menu_border_width == INT_MIN) {
+		theme->menu_border_width = theme->border_width;
+	}
+	if (theme->menu_border_color[0] == FLT_MIN) {
+		memcpy(theme->menu_border_color,
+			theme->window[THEME_ACTIVE].border_color,
+			sizeof(theme->menu_border_color));
+	}
+
 	/* Inherit OSD settings if not set */
 	if (theme->osd_bg_color[0] == FLT_MIN) {
 		memcpy(theme->osd_bg_color,
-			theme->window_active_title_bg_color,
+			theme->window[THEME_ACTIVE].title_bg_color,
 			sizeof(theme->osd_bg_color));
 	}
 	if (theme->osd_border_width == INT_MIN) {
@@ -1512,19 +1508,19 @@ post_processing(struct theme *theme)
 	}
 	if (theme->osd_label_text_color[0] == FLT_MIN) {
 		memcpy(theme->osd_label_text_color,
-			theme->window_active_label_text_color,
+			theme->window[THEME_ACTIVE].label_text_color,
 			sizeof(theme->osd_label_text_color));
 	}
 	if (theme->osd_border_color[0] == FLT_MIN) {
 		/*
-		 * As per http://openbox.org/wiki/Help:Themes#osd.border.color
-		 * we should fall back to window_active_border_color but
-		 * that is usually the same as window_active_title_bg_color
-		 * and thus the fallback for osd_bg_color. Which would mean
+		 * As per https://openbox.org/help/Themes#osd.border.color
+		 * we should fall back to window.active.border.color but
+		 * that is usually the same as window.active.title.bg.color
+		 * and thus the fallback for osd.bg.color. Which would mean
 		 * they are both the same color and thus the border is invisible.
 		 *
-		 * Instead, we fall back to osd_label_text_color which in turn
-		 * falls back to window_active_label_text_color.
+		 * Instead, we fall back to osd.label.text.color which in turn
+		 * falls back to window.active.label.text.color.
 		 */
 		memcpy(theme->osd_border_color, theme->osd_label_text_color,
 			sizeof(theme->osd_border_color));
@@ -1606,15 +1602,11 @@ theme_finish(struct theme *theme)
 		}
 	}
 
-	zdrop(&theme->corner_top_left_active_normal);
-	zdrop(&theme->corner_top_left_inactive_normal);
-	zdrop(&theme->corner_top_right_active_normal);
-	zdrop(&theme->corner_top_right_inactive_normal);
-
-	zdrop(&theme->shadow_corner_top_active);
-	zdrop(&theme->shadow_corner_bottom_active);
-	zdrop(&theme->shadow_edge_active);
-	zdrop(&theme->shadow_corner_top_inactive);
-	zdrop(&theme->shadow_corner_bottom_inactive);
-	zdrop(&theme->shadow_edge_inactive);
+	for (int active = THEME_INACTIVE; active <= THEME_ACTIVE; active++) {
+		zdrop(&theme->window[active].corner_top_left_normal);
+		zdrop(&theme->window[active].corner_top_right_normal);
+		zdrop(&theme->window[active].shadow_corner_top);
+		zdrop(&theme->window[active].shadow_corner_bottom);
+		zdrop(&theme->window[active].shadow_edge);
+	}
 }
