@@ -90,7 +90,7 @@ _osd_update(struct server *server)
 			continue;
 		}
 
-		cairo = buffer->cairo;
+		cairo = cairo_create(buffer->surface);
 
 		/* Background */
 		set_cairo_color(cairo, theme->osd_bg_color);
@@ -150,6 +150,7 @@ _osd_update(struct server *server)
 		g_object_unref(layout);
 		surface = cairo_get_target(cairo);
 		cairo_surface_flush(surface);
+		cairo_destroy(cairo);
 
 		if (!output->workspace_osd) {
 			output->workspace_osd = wlr_scene_buffer_create(
@@ -323,7 +324,7 @@ workspaces_switch_to(struct workspace *target, bool update_focus)
 	struct view *view;
 	enum lab_view_criteria criteria =
 		LAB_VIEW_CRITERIA_CURRENT_WORKSPACE;
-	for_each_view(view, &server->views, criteria) {
+	for_each_view_reverse(view, &server->views, criteria) {
 		if (view->visible_on_all_workspaces) {
 			view_move_to_workspace(view, target);
 		}
@@ -339,12 +340,19 @@ workspaces_switch_to(struct workspace *target, bool update_focus)
 	server->workspaces.current = target;
 
 	/*
-	 * Make sure we are focusing what the user sees.
-	 * Only refocus if the focus is not already on an always-on-top view.
+	 * Make sure we are focusing what the user sees. Only refocus if
+	 * the focus is not already on an omnipresent or always-on-top view.
+	 *
+	 * TODO: Decouple always-on-top views from the omnipresent state.
+	 *       One option for that would be to create a new scene tree
+	 *       as child of every workspace tree and then reparent a-o-t
+	 *       windows to that one. Combined with adjusting the condition
+	 *       below that should take care of the issue.
 	 */
 	if (update_focus) {
 		struct view *view = server->active_view;
-		if (!view || !view_is_always_on_top(view)) {
+		if (!view || (!view->visible_on_all_workspaces
+				&& !view_is_always_on_top(view))) {
 			desktop_focus_topmost_view(server);
 		}
 	}
